//
// Header file defining everything one would ever need to
// deal with the opcodes and instructions for the PIC18F452
// on a machine code level. Woo, doggies. o.o
//

#ifndef _PICOPS_H_INCLUDED
#define _PICOPS_H_INCLUDED


// Instruction types. In one freakin union to keep them all straight, and so
// I don't have to define four freakin' types or whatnot..
typedef union _INSTR {
	unsigned short word;  // Instruction word - some instructions are just this.

	// Literal instructions - branch and addition
	struct {
		unsigned int opcode:4;
		unsigned int value:12;
	} litop4;
	struct {
		unsigned int opcode:5;
		unsigned int value:11;
	} litop5;
	struct {
		unsigned int opcode:8;
		unsigned int value:8;
	} litop8;
	struct {
		unsigned int opcode:12;
		unsigned int value:4;
	} litop12;

	// CALL instruction format
	struct {
		unsigned int opcode:7;
		unsigned int fast:1;
		unsigned int address:8;
	} call;

	// Return instructions
	struct {
		unsigned int opcode:15;
		unsigned int fast:1;
	} ret;

	// The LFSR stuff..(2 word instruction)
	struct {
		unsigned int opcode:10;
		unsigned int fsr:2;
		unsigned int address:4;
	} lfsr;

	// Bit oriented instructions
	struct {
		unsigned int opcode:4;
		unsigned int bits:3;
		unsigned int access:1;
		unsigned int address:8;
	} bit4;

	// Most mathematical instructions use these formats
	struct {
		unsigned int opcode:6;
		unsigned int dest:1;
		unsigned int access:1;
		unsigned int address:8;
	} addop6;

	struct {
		unsigned int opcode:7;
		unsigned int access:1;
		unsigned int address:8;
	} addop7;
	
} INSTR;

// Internal instruction definition
typedef struct _INST_DATA {
	int opcode;
	int length;
	int bytes;
	char* name;
} INST_DATA;

// Opcode lengths
#define FOUR_BIT		0
#define FIVE_BIT		1
#define SIX_BIT			2
#define SEVEN_BIT		3
#define EIGHT_BIT		4
#define FIFTEEN_BIT		5
#define SIXTEEN_BIT		6

// And here are all of the freakin' opcodes. Yay.

// Byte oriented operations (ordered by opcode length)

#define MOVFF	0xC			// 1100

#define DECF	0x1			// 000001
#define IORWF	0x4			// 000100
#define ANDWF	0x5			// 000101
#define XORWF	0x6			// 000110
#define COMF	0x7			// 000111
#define ADDWFC	0x8			// 001000
#define	ADDWF	0x9			// 001001
#define INCF	0xA			// 001010
#define DECFSZ	0xB			// 001011
#define RRCF	0xC			// 001100
#define RLCF	0xD			// 001101
#define SWAPF	0xE			// 001110
#define INCFSZ	0xF			// 001111
#define RRNCF	0x10		// 010000
#define RLNCF	0x11		// 010001
#define INFSNZ	0x12		// 010010
#define DCFSNZ	0x13		// 010011
#define MOVF	0x14		// 010100
#define SUBFWB	0x15		// 010101
#define SUBWFB	0x16		// 010110
#define SUBWF	0x17		// 010111

#define MULWF	0x1			// 0000001
#define CPFSLT	0x30		// 0110000
#define CPFSEQ	0x31		// 0110001
#define CPFSGT	0x32		// 0110010
#define TSTFSZ	0x33		// 0110011
#define SETF	0x34		// 0110100
#define CLRF	0x35		// 0110101
#define NEGF	0x36		// 0110110
#define MOVWF	0x37		// 0110111

// Bit-oriented operations
#define BTG		0x7		// 0111
#define BSF		0x8		// 1000
#define BCF		0x9		// 1001
#define BTFSS	0xA		// 1010
#define BTFSC	0xB		// 1011



// Control operations
#define NOP2	0xF			// 1111

#define BRA		0x1A		// 11010
#define RCALL	0x1B		// 11011

#define BZ		0xE0		// 11100000
#define BNZ		0xE1		// 11100001
#define BC		0xE2		// 11100010
#define BNC		0xE3		// 11100011
#define BOV		0xE4		// 11100100
#define BNOV	0xE5		// 11100101
#define BN		0xE6		// 11100110
#define BNN		0xE7		// 11100111
#define CALL	0xEE		// 11101110
#define GOTO	0xEF		// 11101111
#define RESET	0xFF		// 11111111

#define RETFIE	0x008		// 000000000001000
#define RETURN	0x009		// 000000000001001

#define NOP		0x0000		// 0000000000000000
#define SLEEP	0x0003		// 0000000000000011
#define CLRWDT	0x0004		// 0000000000000100
#define PUSH	0x0005		// 0000000000000101
#define POP		0x0006		// 0000000000000110
#define DAW		0x0007		// 0000000000000111

// Literal operations

#define SUBLW	0x08		// 00001000
#define IORLW	0x09		// 00001001
#define XORLW	0x0A    	// 00001010
#define ANDLW	0x0B		// 00001011
#define RETLW	0x0C		// 00001100
#define MULLW	0x0D		// 00001101
#define MOVLW	0x0E		// 00001110
#define ADDLW	0x0F		// 00001111

#define LFSR	0x3B8		// 1110111000

#define MOVLB	0x10		// 000000010000

// Data memory <-> Program memory
#define TBLRD_STAR			0x0008	// 0000000000001000
#define TBLRD_STAR_PLUS		0x0009	// 0000000000001001
#define TBLRD_STAR_MINUS	0x000A	// 0000000000001010
#define TBLRD_PLUS_STAR		0x000B	// 0000000000001011
#define TBLWT_STAR			0x000C	// 0000000000001100
#define TBLWT_STAR_PLUS		0x000D	// 0000000000001101
#define TBLWT_STAR_MINUS	0x000E	// 0000000000001110
#define TBLWT_PLUS_STAR		0x000F	// 0000000000001111

#endif // _PICOPS_H_INCLUDED